@@ -9,8 +9,11 @@ use dunce::simplified;
 use path_slash::PathExt;
 
 use crate::{
-    artifacts_emitter::ArtifactEmitter, error::BuildError, util::get_artifacts_dir, BuildResult,
-    FileOperation, IOResultExt,
+    artifacts_emitter::ArtifactsEmitter,
+    error::BuildError,
+    plugins::Plugins,
+    util::{get_artifacts_dir, run_command},
+    BuildResult, FileOperation, IOResultExt,
 };
 
 // User configurable options during flutter build
@@ -33,12 +36,14 @@ impl Default for FlutterOptions {
     }
 }
 
+#[derive(Debug)]
 pub enum TargetOS {
     Mac,
     Windows,
     Linux,
 }
 
+#[derive(Debug)]
 pub struct Flutter {
     pub(super) root_dir: PathBuf,
     pub(super) out_dir: PathBuf,
@@ -70,14 +75,14 @@ impl Flutter {
     fn do_flutter_pub_get(&self) -> BuildResult<()> {
         let mut command = self.create_flutter_command();
         command.arg("pub").arg("get");
-        self.run_command(command)
+        self.run_flutter_command(command)
     }
 
     fn do_build(&self) -> BuildResult<()> {
         let flutter_out_root = self.out_dir.join("flutter");
         let flutter_out_dart_tool = flutter_out_root.join(".dart_tool");
         fs::create_dir_all(&flutter_out_dart_tool)
-            .wrap_error(FileOperation::CreateDir, flutter_out_dart_tool.clone())?;
+            .wrap_error(FileOperation::CreateDir, || flutter_out_dart_tool.clone())?;
 
         let package_config = self.root_dir.join(".dart_tool").join("package_config.json");
         let package_config_out = flutter_out_dart_tool.join("package_config.json");
@@ -156,6 +161,18 @@ impl Flutter {
         }
     }
 
+    pub(crate) fn macosx_deployment_target() -> String {
+        match Flutter::target_os() {
+            TargetOS::Mac => {
+                // FIXME: This needs better default
+                std::env::var("MACOSX_DEPLOYMENT_TARGET").unwrap_or_else(|_| "10.13".into())
+            }
+            _ => {
+                panic!("Deployment target can only be called on Mac")
+            }
+        }
+    }
+
     fn target_os() -> TargetOS {
         let target_os = std::env::var("CARGO_CFG_TARGET_OS");
         match target_os.as_deref() {
@@ -173,7 +190,7 @@ impl Flutter {
         local_roots: &mut HashSet<PathBuf>,
     ) -> BuildResult<()> {
         let string =
-            fs::read_to_string(&new).wrap_error(FileOperation::Read, new.as_ref().into())?;
+            fs::read_to_string(&new).wrap_error(FileOperation::Read, || new.as_ref().into())?;
 
         let mut package_config: PackageConfig =
             serde_json::from_str(&string).map_err(|e| BuildError::JsonError {
@@ -188,7 +205,7 @@ impl Flutter {
 
                 let absolute = absolute
                     .canonicalize()
-                    .wrap_error(FileOperation::Canonicalize, absolute)?;
+                    .wrap_error(FileOperation::Canonicalize, || absolute)?;
 
                 {
                     // joining posix path with windows path results in posix
@@ -215,7 +232,7 @@ impl Flutter {
                 source: e,
             })?;
 
-        fs::write(&new, &serialized).wrap_error(FileOperation::Write, new.as_ref().into())?;
+        fs::write(&new, &serialized).wrap_error(FileOperation::Write, || new.as_ref().into())?;
 
         Ok(())
     }
@@ -230,21 +247,8 @@ impl Flutter {
         }
     }
 
-    fn run_command(&self, mut command: Command) -> BuildResult<()> {
-        let output = command
-            .output()
-            .wrap_error(FileOperation::Command, "flutter".into())?;
-
-        if !output.status.success() {
-            Err(BuildError::FlutterToolError {
-                command: format!("{:?}", command),
-                status: output.status,
-                stderr: String::from_utf8_lossy(&output.stderr).into(),
-                stdout: String::from_utf8_lossy(&output.stdout).into(),
-            })
-        } else {
-            Ok(())
-        }
+    fn run_flutter_command(&self, command: Command) -> BuildResult<()> {
+        run_command(command, "flutter")
     }
 
     fn run_flutter_assemble<PathRef: AsRef<Path>>(&self, working_dir: PathRef) -> BuildResult<()> {
@@ -300,7 +304,7 @@ impl Flutter {
             .arg("--suppress-analytics")
             .args(actions);
 
-        self.run_command(command)
+        self.run_flutter_command(command)
     }
 
     fn emit_flutter_artifacts<PathRef: AsRef<Path>>(
@@ -309,7 +313,10 @@ impl Flutter {
     ) -> BuildResult<()> {
         let artifacts_dir = get_artifacts_dir()?;
         let flutter_out_root = self.out_dir.join("flutter");
-        let emitter = ArtifactEmitter::new(&self, flutter_out_root, artifacts_dir)?;
+        let emitter = ArtifactsEmitter::new(&self, flutter_out_root, artifacts_dir)?;
+
+        let plugins = Plugins::new(&self, &emitter);
+        plugins.process()?;
 
         match self.target_os {
             TargetOS::Mac => {
@@ -350,11 +357,11 @@ impl Flutter {
     }
 
     fn emit_checks_for_dir(&self, path: &Path) -> BuildResult<()> {
-        for entry in fs::read_dir(path).wrap_error(FileOperation::ReadDir, path.into())? {
-            let entry = entry.wrap_error(FileOperation::ReadDir, path.into())?;
+        for entry in fs::read_dir(path).wrap_error(FileOperation::ReadDir, || path.into())? {
+            let entry = entry.wrap_error(FileOperation::ReadDir, || path.into())?;
             let metadata = entry
                 .metadata()
-                .wrap_error(FileOperation::MetaData, entry.path())?;
+                .wrap_error(FileOperation::MetaData, || entry.path())?;
             if metadata.is_dir() {
                 self.emit_checks_for_dir(entry.path().as_path())?;
             } else {
@@ -369,8 +376,8 @@ impl Flutter {
     fn copy<P: AsRef<Path>, Q: AsRef<Path>>(from: P, to: Q) -> Result<u64, BuildError> {
         fs::copy(&from, &to).wrap_error_with_src(
             FileOperation::Copy,
-            to.as_ref().into(),
-            from.as_ref().into(),
+            || to.as_ref().into(),
+            || from.as_ref().into(),
         )
     }
 }
